import plotext as plt

def plot_bars(x, y, title='', width=200, ylabel=[], color=''):
    if color == '':
        color = 'white'
    ylabel = [' ' + label for label in ylabel]
    x = [i + '  ' for i in x]
    plt.bar(x, y, orientation="horizontal", width = 1 / 100, color=color, marker = "▇")
    plt.title(title)

    [plt.text(
        ylabel[i], x = y[i], y = i + 1,
        alignment = 'left', color=color, style="bold", background="default", orientation="h",
    ) for i in range(len(x))]
    plt.xaxes(False, False)
    plt.yaxes(False, False)
    plt.axes_color('transparent')
    plt.xlim(0, max(y) + 2 * max([len(label) for label in ylabel]) + 10)
    plt.plot_size(width, len(x) + int(len(title) > 0))
    plt.theme('clear')
    plt.ticks_style('bold')
    plt.xfrequency(0)
    plt.yfrequency(0)
    plt.show()



import time
import threading
import sys


def format_duration(seconds):
    minutes, seconds = divmod(int(seconds), 60)
    hours, minutes = divmod(minutes, 60)
    if hours:
        return f"{hours}h {minutes}m {seconds}s"
    elif minutes:
        return f"{minutes}m {seconds}s"
    else:
        return f"{seconds}s"


def timer_display(start, stop_event):
    while not stop_event.is_set():
        elapsed = time.time() - start
        sys.stdout.write(f"\r{format_duration(elapsed)}")
        sys.stdout.flush()
        time.sleep(1)


def timer_start(label):
    start = time.time()
    stop_event = threading.Event()
    thread = threading.Thread(target=timer_display, args=(start, stop_event), daemon=True)
    thread.start()

    try:
        input()
    except KeyboardInterrupt:
        print("\nCtrl+C Interrupt!")
    finally:
        stop_event.set()
        thread.join(timeout=0.1)

    end = time.time()
    duration = end - start
    return start, end, duration



from datetime import datetime, timedelta


def iso_datetime(ts):
    return (ts).strftime("%Y-%m-%d %H:%M:%S")


def iso_date(ts):
    return f'{(ts).strftime("%Y-%m-%d")} 00:00:00'


def timestamp_from_unix_ts(ts):
    return iso_datetime(datetime.fromtimestamp(ts))


def today():
    return iso_date(datetime.today())


def yesterday():
    return iso_date(datetime.today() - timedelta(days=1))


def monday_of_this_week():
    return iso_date(datetime.today() - timedelta(days=datetime.today().weekday()))


def first_of_this_month():
    return iso_date(datetime.today().replace(day=1))


def next_day_from_iso(iso_ts):
    return iso_datetime(
        datetime.strptime(iso_ts, "%Y-%m-%d %H:%M:%S") + timedelta(days=1)
    )


def timestamp_normalize(ts):
    match ts:
        case "today":
            return today()
        case "yesterday":
            return yesterday()
        case "monday":
            return monday_of_this_week()
        case "first":
            return first_of_this_month()
        case _:
            return iso_date(datetime.strptime(ts, "%Y.%m.%d"))



import sqlite3
from pathlib import Path

DATABASE_PATH = Path.home() / ".pyflo.db"


def database():
    connection = sqlite3.connect(DATABASE_PATH)
    return connection


def init():
    connection = database()
    cursor = connection.cursor()
    cursor.execute(
        """CREATE TABLE IF NOT EXISTS log(
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        label TEXT NOT NULL,
        start_ts TEXT NOT NULL,
        end_ts TEXT NOT NULL
    )"""
    )
    connection.commit()
    connection.close()


def database_log(label, start_ts, end_ts):
    connection = database()
    cursor = connection.cursor()
    cursor.execute(
        """
        INSERT INTO log (label, start_ts, end_ts)
        VALUES (?, ?, ?)
    """,
        (label, start_ts, end_ts),
    )
    connection.commit()
    connection.close()


def database_duration_query(start_ts, end_ts, by, filters):
    connection = database()
    cursor = connection.cursor()
    between = "WHERE start_ts >= ? AND end_ts <= ?"
    filter = (
        f"AND label IN ({', '.join('?' for _ in filters)})" if len(filters) > 0 else ""
    )
    duration_seconds = """
        SUM(
            strftime('%s', end_ts) - strftime('%s', start_ts)
        ) AS duration_seconds
    """
    if by == "week":
        query_string = f"""
        SELECT strftime('%w', start_ts) AS weekday, {duration_seconds}
        FROM log
        {between} {filter}
        GROUP BY weekday
        ORDER BY weekday
        """
    if by == "day":
        query_string = f"""
        SELECT strftime('%H', start_ts) AS hour, {duration_seconds}
        FROM log
        {between} {filter}
        GROUP BY hour
        ORDER BY hour
        """
    if by == "label":
        query_string = f"""
        SELECT label, {duration_seconds}
        FROM log
        {between} {filter}
        GROUP BY label
        ORDER BY duration_seconds DESC
        """
    args = (start_ts, end_ts, *filters)
    result = cursor.execute(query_string, args)
    return normalize_duration_query_result(result.fetchall(), by)


def normalize_duration_query_result(data, by):
    weekdays = [
        "Sunday",
        "Monday",
        "Tuesday",
        "Wednesday",
        "Thursday",
        "Friday",
        "Saturday",
    ]
    if by == "week":
        l = [(x, 0) for x in weekdays]
    if by == "day":
        l = [(f"{x:02}:00", 0) for x in range(24)]
    if by == "label":
        return data
    for i, x in enumerate(data):
        index = int(x[0])
        l[index] = (l[index][0], x[1])
    return l


def database_list_labels(filters):
    connection = database()
    cursor = connection.cursor()
    where = " OR ".join(["label LIKE ?"] * len(filters))
    where = "WHERE " + where if len(filters) > 0 else ""
    result = cursor.execute(
        f"SELECT label FROM log {where}", tuple(f"%{x}%" for x in filters)
    )
    return "\n".join([x[0] for x in result.fetchall()])


init()



import click
from pyfloq import __version__
from pyfloq.chart import plot_bars
from pyfloq.timer import timer_start, format_duration
from pyfloq.database_handler import database_log, database_duration_query, database_list_labels
from pyfloq.timestamps import timestamp_normalize, timestamp_from_unix_ts, next_day_from_iso


@click.group()
@click.version_option(__version__)
def pyflo():
    pass


@pyflo.command()
@click.argument("label")
def track(label):
    click.echo(f"tracking {label}, press Return to stop.")
    start, stop, duration = timer_start(label)
    start, stop = timestamp_from_unix_ts(start), timestamp_from_unix_ts(stop)
    database_log(label, start, stop)
    click.echo(f"{format_duration(duration)} logged under {label}.")


@pyflo.command()
@click.argument("when")
@click.option("--to", help="specify another date to analyze a range")
@click.option(
    "--by",
    type=click.Choice(["day", "week", "label"]),
    default="week",
    show_default=True,
    required=True,
    help="grouping for the data, presented as separate bars",
)
@click.option("--filter", "-f", multiple=True, help="selective filtering for labels")
@click.option("--color", help="color for bars")
def vis(when, to, by, filter, color):
    when = timestamp_normalize(when)
    if to is None:
        to = next_day_from_iso(when)
    when, to = sorted([when, to])
    data = database_duration_query(when, to, by, filter)
    data.reverse()
    if len(data) == 0:
        click.echo('Nothing tracked in this time period.')
        return
    x, y, ylabel = zip(*[(x, float(y), format_duration(y)) for x, y in data])
    plot_bars(x, y, ylabel=ylabel, title='', color=color)


@pyflo.command()
@click.option("--filter", "-f", multiple=True, help="selective filtering")
def list(filter):
    click.echo(database_list_labels(filter))


if __name__ == "__main__":
    pyflo()
